// -*- C++ -*- 
// file:   PIMSGen.cc
// author: Emiliano Necciari
// email:  necciari@di.unipi.it
//
//
// With this software we can generate some instances of machine 
// scheduling problems.
//
// In the file "problems" we can find the number  and the type 
// of instances that this software will create
// The format of file "problems" is:
// 
//        p
//        m1 n1
//        m2 n2
//          .
//          .
//          .
//        mp np
// where:
//        p  - is the number of instances 
//        mi - is the number of machines of instance i
//        ni - is the number of jobs of instance i
//
// All processing times of jobs are chosen in the interval
// [rangeMin,rangeMax].
// In particular, we chose the processing times of 1% of 
// jobs in the interval [rangeMin,(rangeMax-rangeMin)*0.2],
// and we take the remaining processing times in  the interval
// [(rangeMax-rangeMin)*0.9,rnageMax].


#include <iostream>
#include <fstream>
#include <getopt.h>
#include <cmath>
#include <cstdlib>


#define  println(x) cout<<x<<endl
#define  print(x) cout<<x

extern "C" {
#include "rng.h"
}

int main( int argc, char** argv ){

  int numJobs    = 10; // default number of jobs
  int numMachine = 5;  // default number of machines
  int rangeMax   = 100;// default max processing time of a job
  int rangeMin   = 1;  // default min processing time of a job
  int seed       = 1;  // default seed
  int numProb;        
    
  //
  // Parsing of command line arguments
  //
  int opt;
  while( ( opt = getopt(argc,argv,"-a:b:s:") ) != -1 )
    switch( opt ){
    case 'a': rangeMin    = atoi(optarg);break;
    case 'b': rangeMax    = atoi(optarg);break;
    case 's': seed        = atoi(optarg);break;
    case '?':
      cout<<endl<<"usage:\n     "<<argv[0]
	  <<" -aA -bB -sSEED"<<endl
	  <<"\n\t A is the min of range jobs duration"
	  <<endl
	  <<"\t B is the max of range "
	  <<endl
	  <<"\t SEED is the initial seed"
	  <<endl
	  <<endl;
      return 1;
    }

  //
  // Opening file "problems"
  //
  ifstream ip("problems");
  
  if( !ip ){
    cerr<<"Error on opening file: problems"<<endl;
    return 1;
  }
  
  //
  // Setting the seed
  //
  setseed(seed);

  
  //
  // Reading the number of instances that we
  // must create
  //
  ip>>numProb;
  
  //
  // Printing the number of instances that we
  // will create
  //
  println(numProb);
  
  
  int delta  = rangeMax-rangeMin; 
  
  int cIrMax = rangeMax;
  int cIrMin = (int)ceil(delta*0.9);
  
  int cIIrMax = (int)ceil(delta*0.2);
  int cIIrMin = rangeMin;
  
  for( ; --numProb >= 0 ;){
    
    //
    // Reading the number of machines and jobs 
    // of the instance that we must create
    //
    ip>>numMachine>>numJobs;
    
    //
    // Printing the number of machines and jobs 
    // of the instance that we will create
    //
    println(numJobs<<" "<<numMachine);
    
    int nLcII = (int)ceil(numJobs*0.01);
    int nLcI  = numJobs - nLcII;
  
    
    for( int i = 0 ; i < nLcI ; i++ )
      println(iunifab(cIrMin,cIrMax));
      
    
    for( int i = 0 ; i < nLcII ; i++ )
      println(iunifab(cIIrMin,cIIrMax));
    
  }
  return 0;
}







