/* **   ***********************************************************   ** 
   **                                                                 **
   **                                                                 **
   **     XXXXXXXXXXXX    Random Number Generator     XXXXXXXXXXXX    **
   **     XXX                                                  XXX    **
   **     XXX    The program is based on the book              XXX    **
   **     XXX    BRATLEY P.,FOX B.L.,SCHRAGE L.E.,             XXX    **
   **     XXX    "A Guide to simulation",                      XXX    **
   **     XXX    Springler-Verlag,Ney-York,1983.               XXX    **
   **     XXX                                                  XXX    **
   **     XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX    **
   **                                                                 **
   **      Portable random number generator using the recursion :     **
   **      SEED = 16807 * SEED mod (2**(31)-1)                        **
   **      using only 32 bits, including sign.                        **
   **                                                                 **
   **   ***********************************************************   ** */
#define      M    2147483647
static int   SEED=1;

/* **   ***********************************************************   ** 
   **                                                                 **
   **                                                                 **
   **   SYNOPSIS                                                      **
   **   --------                                                      **
   **   int    setseed(seed)                                          **
   **   int    seed;                                                  **
   **                                                                 **
   **   INPUT PARAMETER                                               **
   **   ---------------                                               **
   **   seed = Integer greater than 0 and less than 2147483647        **
   **                                                                 **
   **   DESCRIPTION                                                   **
   **   -----------                                                   **
   **   1) Set seed                                                   **
   **   2) setseed=oldseed                                            **
   **                                                                 **
   **   ***********************************************************   ** */
int    setseed(int seed)
{
 int   oldseed;

 oldseed=SEED;
 if (seed>0) { SEED=seed; }
 return (oldseed);
}

/* **   ***********************************************************   ** 
   **                                                                 **
   **                                                                 **
   **   SYNOPSIS                                                      **
   **   --------                                                      **
   **   float  unif()                                                 **
   **                                                                 **
   **   DESCRIPTION                                                   **
   **   -----------                                                   **
   **   1) Portable random number generator using the recursion :     **
   **      SEED = 16807 * SEED mod (2**(31)-1)                        **
   **      using only 32 bits, including sign.                        **
   **   2) unif = A uniform fraction between 0 and 1.                 **
   **                                                                 **
   **   ***********************************************************   ** */
float  unif(void)
{
 int   k;
 double u;
 
 k=SEED/127773;
 SEED=16807*(SEED-k*127773)-k*2836;
 if (SEED<0) { SEED+=2147483647; }
 u=SEED*4.656612875E-10;
 return((float)u);
}

/* **   ***********************************************************   ** 
   **                                                                 **
   **                                                                 **
   **   SYNOPSIS                                                      **
   **   --------                                                      **
   **   int    irand()                                                **
   **                                                                 **
   **   DESCRIPTION                                                   **
   **   -----------                                                   **
   **   1) Portable random number generator using the recursion :     **
   **      SEED = 16807 * SEED mod (2**(31)-1)                        **
   **      using only 32 bits, including sign.                        **
   **   2) irand = A random number between 0 and 2147483647.          **
   **                                                                 **
   **   ***********************************************************   ** */
int    irand(void)
{
 int   k;

 /*printf("SEED %d\n", SEED);*/
 
 k=SEED/127773;
 SEED=16807*(SEED-k*127773)-k*2836;
 if (SEED<0) { SEED+=2147483647; }
 return(SEED);
}

/* **   ***********************************************************   ** 
   **                                                                 **
   **                                                                 **
   **   SYNOPSIS                                                      **
   **   --------                                                      **
   **   int    iunif(n)                                               **
   **   int    n;                                                     **
   **                                                                 **
   **   DESCRIPTION                                                   **
   **   -----------                                                   **
   **   1) Portable code to generate an integer uniform over the      **
   **      interval 1,n using inversion.                              **
   **      Essentially, it is an infinite precision implementation    **
   **      of the method iunif = n * unif()+1 which avoids the        **
   **      problems due to finite floating point arithmetic.          **
   **      The random generator is assumed to use this modulus :      **
   **      M=2147483647  (2**31)-1)                                   **
   **   2) iunif = An integer uniform over the interval 1,n.          **
   **                                                                 **
   **   ***********************************************************   ** */
int    iunif(int n)
{
 int  ix,jtry,i,j,k,l,iji,kli,ijr,klr;
 
 extern int irand();


      ix=irand();

      /*printf("ix %d M %d  n %d \n", ix, M, n);*/

      jtry=ix/(M/n);       /* We want integer part of ix/(M/n) using infinite */
   
      /*printf("jtry %d \n", jtry);*/

      if(jtry<=0)goto FIN; /* precision. Decrease jtry until jtry <= ix/(M/n) */
      else       goto BBB; /* i.e.  M/ix <= n/jtry.                           */
 AAA: jtry--; 

 BBB: i=M;                  /* Now check if M/ix <= n/jtry */
      j=ix;
      k=n;
      l=jtry;
                           /* More generally, is i/j <= k/l, where i>=j, k>=l */
 CCC: iji=i/j;             /* If the integer parts differ, the question is    */
      kli=k/l;             /* answered.                                       */
      if(iji<kli)goto FIN;
      if(iji>kli)goto AAA;
                           /* Must look at the remainder;                     */
      ijr=i-iji*j;         /* Is ijr/j <= klr/l                               */
      if (ijr<=0)goto FIN;
      klr=k-kli*l;
      if (klr<=0)goto AAA;
                           /* Still not resolved;                             */
      i=l;                 /* Rephrase in standard form,                      */
      k=j;                 /* i.e.  is  l/klr <= j/ijr ?                      */
      j=klr;
      l=ijr;
      goto CCC;
                           /* Now add 1 to put in the range 1,n               */
 FIN: return(jtry+1);
}

/* **   ***********************************************************   ** 
   **                                                                 **
   **                                                                 **
   **   SYNOPSIS                                                      **
   **   --------                                                      **
   **   int    iunifab(a,b)                                           **
   **   int    a,b;                                                   **
   **                                                                 **
   **   DESCRIPTION                                                   **
   **   -----------                                                   **
   **   1) iunifab = An integer uniform over the interval a,b.        **
   **                                                                 **
   **   ***********************************************************   ** */
int    iunifab(int a, int b)
{
 extern int iunif();

 if (a<b)      { return (a-1+iunif(b-a+1)); }
 else if(a==b) { return (a); }
 else          { return (0); }
}

/* **   ***********************************************************   ** 
   **                                                                 **
   **                                                                 **
   **   SYNOPSIS                                                      **
   **   --------                                                      **
   **   float  expn(lambda)                                           **
   **   float  lambda;                                                **
   **                                                                 **
   **   DESCRIPTION                                                   **
   **   -----------                                                   **
   **   1) expn = An exponential random number.                       **
   **   2) using the transformation:                                  **
   **          -LOG(1-U)                                              **
   **      X = ---------                                              **
   **           LAMBDA                                                **
   **                                                                 **
   **   ***********************************************************   ** */
float  expn(float lambda)
{
 extern float unif();

 if(lambda<=0.0) return(0.0);
 else return( ( 0.0-log( 1.0-unif() ) )/lambda );
}

/* **   ***********************************************************   ** 
   **                                                                 **
   **                                                                 **
   **   SYNOPSIS                                                      **
   **   --------                                                      **
   **   int    poisson(theta)                                         **
   **   float  theta;                                                 **
   **                                                                 **
   **   DESCRIPTION                                                   **
   **   -----------                                                   **
   **   1) poisson = An integer POISSON number.                       **
   **   2) using the generator:                                       **
   **      Set X= -1, m=EXP(theta).                                   **
   **      Repeat the following until m<1:                            **
   **             Generate U uniform on (0,1),                        **
   **             Set X=X+1, m=m*U.                                   **
   **      Output X.                                                  **
   **                                                                 **
   **   ***********************************************************   ** */
int    poisson(float theta)
{
 int x;
 float m;
 extern float unif();

 x= -1;
 m=exp(theta);

 do {x+=1;
     m=m*unif();
    }while(m>1.0);

 return(x);
}
